#ifndef __PLUGIN_LAYER_H__
#define __PLUGIN_LAYER_H__

#include <cassert>
#include <chrono>
#include <cstring>
#include <memory>
#include <iostream>
#include <cublas_v2.h>
#include <cudnn.h>
#include <iostream>
#include <sstream>
#include <string.h>
#include <cstring>
#include <time.h>
#include <vector>
#include <cuda_runtime.h>
#include <cuda.h>

#include "NvInferPlugin.h"
#include "NvUffParser.h"

using namespace nvinfer1;
using namespace nvuffparser;
using namespace plugin;

#define CHECK(status)                                                                                           \
    {                                                                                                                           \
        if (status != 0)                                                                                                \
        {                                                                                                                               \
            std::cout << "Cuda failure: " << cudaGetErrorString(status) \
                      << " at line " << __LINE__                                                        \
                      << std::endl;                                                                     \
            abort();                                                                                                    \
        }                                                                                                                               \
    }


class Split : public IPlugin
{
public:

    Split();
    Split(const void* buffer, size_t length);

    int getNbOutputs()  const override;
    Dims getOutputDimensions(int index, const Dims* inputs, int nbInputDims) override;

    void configure(const Dims*inputs, int nbInputs, const Dims* outputs, int nbOutputs, int) override;
    inline size_t getWorkspaceSize(int) const override { return 0; }

    int initialize() override;
    int enqueue(int batchSize, const void* const* inputs, void** outputs, void*, cudaStream_t stream) override;
    inline void terminate() override;

    size_t getSerializationSize() override;
    void serialize(void* buffer) override;


protected:
    template<typename T> void write(char*& buffer, const T& val)
    {
        *reinterpret_cast<T*>(buffer) = val;
        buffer += sizeof(T);
    }

    template<typename T> void read(const char*& buffer, T& val)
    {
        val = *reinterpret_cast<const T*>(buffer);
        buffer += sizeof(T);
    }

    int mAxis = 3;
    int mNumSplit = 3;
    Dims3 mOutputDims;
    Dims3 mInputDims;
};



class PluginFactory : public nvinfer1::IPluginFactory, public nvuffparser::IPluginFactory
{
public:
    virtual nvinfer1::IPlugin* createPlugin(const char* layerName, const nvinfer1::Weights* weights, int nbWeights, const nvuffparser::FieldCollection fc) override;
    IPlugin* createPlugin(const char* layerName, const void* serialData, size_t serialLength) override;
    bool isPlugin(const char* name) override;
    void destroyPlugin();
    std::unique_ptr<Split> mSplit{nullptr};
};

#endif